﻿//{$WARNINGS OFF}
//{$WARN SYMBOL_PLATFORM OFF}
//{$WARN UNIT_PLATFORM OFF}
//=============================================================================
//  テキストを Base64 で符号化して保存
//
//-----------------------------------------------------------------------------
//
//  【使用条件等】
//
//  当サイトのアプリケーション類，コンポーネント，コード類は，転載，改変，配布
//  (オリジナル，改変版)などは全て自由です．商用利用も自由です．連絡，使用の記
//  載義務もありません．ただしオリジナルそのものを商用とすることは禁止します．
//  なお，本サイトのプログラムやコードを使用したことによって生じた，いかなる障
//  害，損失に関しても作者は一切関与しないものとします．ご了承下さい．
//
//-----------------------------------------------------------------------------
//
//  【履歴】
//
//  2017年01月18日 ～
//
//-----------------------------------------------------------------------------
//
//  【動作確認環境】
//
//  Windows 7 U64(SP1) + Delphi XE(UP1) Pro
//
//  Presented by Mr.XRAY
//  http://mrxray.on.coocan.jp/
//=============================================================================
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Memo1: TMemo;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private 宣言 }
    FFilePath : string;
  public
    { Public 宣言 }
  end;

var
  Form1: TForm1;

implementation

uses
  EncdDecd;

{$R *.dfm}

//=============================================================================
//  フォーム生成時の処理
//  FFilePath は String 型のグローバル変数
//=============================================================================
procedure TForm1.FormCreate(Sender: TObject);
begin
  Memo1.Lines.Clear;

  //文字列をセット
  Memo1.Lines.Add(#$4F60#$597D + ' !!');
  Memo1.Lines.Add(#$D842#$DFB7 + ' (サロゲートペアな文字)');
  Memo1.Lines.Add('テキストを Base64 で符号化して保存');

  FFilePath := ExpandFileName('..\000.txt');
end;

//=============================================================================
//  テキストを Base64 で符号化 ( Base64 でエンコード ) して保存
//
//  文字列を引数にしてエンコード (EncodeBase64 を使用)
//  uses に EncdDecd が必要
//=============================================================================
procedure TForm1.Button1Click(Sender: TObject);
var
  LUtf8Str   : UTF8String;
  LAnsiStr   : AnsiString;
  LMemStream : TMemoryStream;
begin
  // UTF-8 の文字列にする
  LUtf8Str := Utf8String(Memo1.Lines.Text);

  LMemStream := TMemoryStream.Create;
  try
    // UTF-8 のデータを Base64 に符号化
    // EncodeBase64 関数の戻り値は AnsiString 型
    LAnsiStr := EncodeBase64(Addr(LUtf8Str[1]), Length(LUtf8Str));

    // メモリストリームに符号化した文字列を書き込んで保存
    LMemStream.Clear;
    LMemStream.Write(LAnsiStr[1], Length(LAnsiStr));
    LMemStream.SaveToFile(FFilePath);
  finally
    FreeAndNil(LMemStream);
  end;
end;

//=============================================================================
//  Base64 で符号化して保存したテキストを読み出して複合化 ( Base64 のデコード
//
//  文字列を引数にしてデコード (DecodeBase64 を使用)
//  uses に EncdDecd が必要
//=============================================================================
procedure TForm1.Button2Click(Sender: TObject);
var
  LAnsiStr   : AnsiString;
  LMemStream : TMemoryStream;
  LByteArray : TBytes;
begin
  Memo1.Lines.Clear;

  LMemStream := TMemoryStream.Create;
  try
    // テキストファイルをメモリストリームに読み込む
    LMemStream.LoadFromFile(FFilePath);

    // 文字列の長さを設定
    SetLength(LAnsiStr, LMemStream.Size);

    // メモリストリームから文字列を読み出す
    LMemStream.Position := 0;
    LMemStream.Read(LAnsiStr[1], LMemStream.Size);

    // Base64 の文字列をデコードしてバイト配列に格納
    // DecodeBase64 関数の戻り値はバイト配列
    LByteArray := DecodeBase64(LAnsiStr);

    // バイト配列のデータを文字列のデータにする
    // UTF-8 の指定が必要なければ StringOf メソッドが使用可能
    Memo1.Lines.Text := TEncoding.UTF8.GetString(LByteArray);
  finally
    FreeAndNil(LMemStream);
  end;
end;

end.
